/* Implementazione oggetto client */

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.rmi.Naming;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

public class ClientImpl extends UnicastRemoteObject implements ClientInterface {
	private static final int DEFAULT_PORT = 1099;

	static boolean termina = false;

	static Thread threadMain = null;

	// Costruttore
	public ClientImpl() throws RemoteException {
		super();
	}

	/**
	 * Implementazione metodo remoto
	 */
	public void ricevi_offerta(String bid) throws java.rmi.RemoteException {
		System.out.println("Ricevuta offerta di un client: " + bid);
	}

	/**
	 * Implementazione metodo remoto
	 */
	public void notify_terminazione() throws java.rmi.RemoteException {
		/*
		 * Alla ricezione della prossima offerta dall'utente il client termina
		 */
		termina = true;
		System.out.println("Asta terminata!");
	}

	public static void main(String args[]) {

		String hostName = null;
		int portNum = -1;

		// Controllo dei parametri della riga di comando
		if (args.length != 1 && args.length != 2) {
			System.out.println("Sintassi: ClientImpl NomeHost [registryPort]");
			System.exit(1);
		} else {
			hostName = args[0];
			if (args.length == 2) {
				try {
					portNum = Integer.parseInt(args[1]);
				} catch (Exception e) {
					System.out
							.println("Sintassi: ClientImpl NomeHost [registryPort], registryPort intero");
					System.exit(2);
				}
			} else
				portNum = DEFAULT_PORT;
		}

		try {
			InputStreamReader is = new InputStreamReader(System.in);
			BufferedReader br = new BufferedReader(is);
			String registryURL = "//" + hostName + ":" + portNum + "/astaServer";

			/* recupero riferimento al server */
			ServerInterface server_ref = (ServerInterface) Naming.lookup(registryURL);
			System.out.println("Lookup completed ");
			threadMain = Thread.currentThread();

			/* Istanzio l'oggetto client */
			ClientImpl client_ref = new ClientImpl();

			int result = -4;
			/* ciclo di invio offerte fino alla terminazione dell'asta */
			while (!termina) {
				System.out.println("Nuova bid?");
				int bid = Integer.parseInt(br.readLine());

				/* invio offerta */
				result = server_ref.effettuaOfferta(bid, client_ref);
				switch (result) {
				case 0:
					System.out.println("L'offerta fatta e' vincente!");
					break;
				case -1:
					System.out.println("Non si possono effettuare altre offerte, "+
					"numero massimo di partecipanti raggiunto!");
					break;
				case -2:
					System.out.println("Asta terminata");
					termina=true;
					break;
				case -3:
					System.out.println("Valore minore del valore dell'ultima offerta valida ricevuta");
					break;

				default:
					System.out.println("Offerta di "+result+" euro accettata!");
					break;
				}
			}
			/* Dopo un certo timeout interno di attesa dell'infrastruttura
			 * la JVM dove e' in esecuzione l'oggetto remoto client termina
			 */ 
		} // end try
		catch (Exception e) {
			System.out.println("Exception in ClientImpl: " + e);
			System.exit(1);
		} // end catch
	} //end main

}//end class
