/* Programma server */

import java.rmi.Naming;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.server.UnicastRemoteObject;

public class ServerImpl extends UnicastRemoteObject implements ServerInterface {

	private static final int DEFAULT_PORT = 1099;

	private static final int NUM_MASSIMO_PARTECIPANTI = 30;

	static boolean termina = false;

	static Attesa a = new Attesa();

	static private int numeroPartecipanti;

	static private ClientInterface[] partecipanti = new ClientInterface[NUM_MASSIMO_PARTECIPANTI];

	static int contOfferte = 0;

	static int offertaMassima = 0;

	static boolean stabilitaOffertaMassima = false;

	static int offertaAttuale = 0;

	// Costruttore
	public ServerImpl() throws RemoteException {
		super();
	}

	public int effettuaOfferta(int offerta, ClientInterface client_ref)
			throws java.rmi.RemoteException {
		if (numeroPartecipanti == NUM_MASSIMO_PARTECIPANTI)
			return -1;
		if (termina == true)
			return -2;
		if (offerta < offertaAttuale)
			return -3;
		else
			offertaAttuale = offerta;

		/*
		 * Verifico se il client e' gia' memorizzato nella struttura dati del server
		 */
		int posizione = -1;
		for (int i = 0; i < numeroPartecipanti; i++) {
			if (partecipanti[i].equals(client_ref)) {
				System.out.println("Client gia' registrato nella struttura dati");
				posizione = i;
				break;
			}
		}
		if (posizione == -1) {
			System.out.println("Aggiungo un client alla struttura dati");
			partecipanti[numeroPartecipanti] = client_ref;
			numeroPartecipanti++;
		}

		// Effettuo la callback dell'offerta
		doCallback("" + offerta);

		// Con le prime cinque offerte stabilisco il valore massimo
		if (!stabilitaOffertaMassima && (contOfferte < 5)) {
			offertaMassima = offertaMassima + offerta;
			if (contOfferte == 4) {
				offertaMassima = offertaMassima * 10;
				stabilitaOffertaMassima = true;
				System.out.println("Offerta massima: " + offertaMassima);
			}
			contOfferte++;
		}

		// Controllo la condizione di terminazione
		if (stabilitaOffertaMassima && (offerta >= offertaMassima)) {
			termina = true;
			a.attivaTerminazione();
			return 0;
		}

		return offerta;
	}

	/*
	 * metodo di callback a tutti gli utenti che stanno partecipando all'asta
	 */
	private synchronized void doCallback(String msg) throws RemoteException {
		for (int i = 0; i < numeroPartecipanti; i++) {
			ClientInterface nextClient = partecipanti[i];
			nextClient.ricevi_offerta("" + msg);
		}// end for
	} // end doCallback

	private synchronized static void terminaClient() throws RemoteException {
		// Liberiamo le risorse, eliminando anche tutti i riferimenti ai
		// client
		for (int i = 0; i < numeroPartecipanti; i++) {
			partecipanti[i].notify_terminazione();
			partecipanti[i] = null;
		}
		numeroPartecipanti = 0;
	}

	public static void main(String args[]) {
		String registryURL;
		int RMIPortNum = -1;

		// Controllo dei parametri della riga di comando
		if (args.length != 0 && args.length != 1) {
			System.out.println("Sintassi: ChatServer [registryPort]");
			System.exit(1);
		}
		if (args.length == 1) {
			try {
				RMIPortNum = Integer.parseInt(args[0]);
			} catch (Exception e) {
				System.out
						.println("Sintassi: ChatServer [registryPort], registryPort intero");
				System.exit(2);
			}
		} else
			RMIPortNum = DEFAULT_PORT;

		try {
			startRegistry(RMIPortNum);

			/*
			 * istanziazione del server e pubblicazione sul registry
			 */
			ServerImpl server_ref = new ServerImpl();
			registryURL = "//localhost:" + RMIPortNum + "/astaServer";
			Naming.rebind(registryURL, server_ref);
			a.sospendiInAttesaTerminazione();
			terminaClient();
			System.out.println("Esco dal main del server");
		} // end try
		catch (Exception re) {
			System.out.println("Exception in ServerImpl.main: " + re);
			System.exit(1);
		} // end catch
	} // end main

	/*
	 * metodo che verifica se c gi un registry attivo sullhost e porta
	 * indicati, e se non c lo crea
	 */
	private static void startRegistry(int RMIPortNum) throws RemoteException {
		try {
			Registry registry = LocateRegistry.getRegistry(RMIPortNum);
			registry.list();
		} catch (RemoteException e) {
			LocateRegistry.createRegistry(RMIPortNum);
		}
	} // end startRegistry

} // end class

/*
 * Oggetto utilizzato per sospendere il Thread che esegue il main del server,
 * fino a quando non si verifica la situazione di terminazione
 */
class Attesa {
	protected synchronized void sospendiInAttesaTerminazione() {
		try {
			wait();
		} catch (Exception e) {
			// TODO: handle exception
			System.out.println("Problema, il seguente: " + e);
			e.printStackTrace();
		}
	}

	protected synchronized void attivaTerminazione() {
		notify();
	}
}